import * as THREE from 'https://unpkg.com/three@v0.150.0/build/three.module.js';

var width = 800;
var height = 500;

var renderer, scene, camera;
var planet;
var lightPosition;

var planetVertexShader, planetFragmentShader;

/**
 * Function to tell us the milliseconds
 */
function millis() {
    return (new Date()).getTime();
}

/**
 * Converts degrees to radians
 */
function toRad(degree) {
    return Math.PI * 2 * degree / 360;
}

window.onLoad = async () => {
    var canvasContainer = document.getElementById('myCanvasContainer');

    planetVertexShader = await (await fetch('vertex.glsl')).text();
    planetFragmentShader = await (await fetch('fragment.glsl')).text();

    renderer = new THREE.WebGLRenderer();
    renderer.setSize(width, height);
    canvasContainer.appendChild(renderer.domElement);

    scene = new THREE.Scene();

    // We will use a perspective camera, with FOV 80 degrees, correct aspect ratio, near plane at distance 1, far plane at distance 1000
    camera = new THREE.PerspectiveCamera(80, width / height, 1, 30);
    camera.position.z = 15;
    camera.position.y = 0;
    camera.up = new THREE.Vector3(0, 1, 0);
    camera.lookAt(new THREE.Vector3(0, 0, 0));
    scene.add(camera);

    lightPosition = new THREE.Vector3(50, 40, 20);
    planet = addPlanet();

    draw();
}

function draw() {
    requestAnimationFrame(draw);

    planet.rotation.set(0, toRad((millis() / 50) % 360), 0);

    renderer.render(scene, camera);
}

/**
 * This function creates the planet.
 */
function addPlanet() {
    var planet = new THREE.Object3D();
    var sphere = createSphere();

    sphere.scale.set(7, 7, 7);
    planet.add(sphere);
    scene.add(planet);

    return planet;
}

/**
 * Creates a sphere.
 * --Task-- Finish this function.
 */
function createSphere() {
    // -- Task 1 -- Make the sphere look smoother by changing the geometry values
    var geometry = new THREE.SphereGeometry(1, 8, 6);

    var colorWater = new THREE.Color(0x00aeef);
    var colorAtmosphere = new THREE.Color(0x66d5ed);
    var color1 = new THREE.Color(0x197b30);
    var color2 = new THREE.Color(0x005826);
    var color3 = new THREE.Color(0x4e3f32);

    var material = new THREE.ShaderMaterial({
        uniforms: {
            // -- Task 2 -- Pass light position to the shader
            // -- Task 3 -- Pass planet colors to the shader 
            //	  (You could use the color values defined above but you can also change them to get more interesting planet)
        },
        vertexShader: planetVertexShader,
        fragmentShader: planetFragmentShader
    });
    var sphere = new THREE.Mesh(geometry, material); // We create our cube with that material

    return sphere;
}